=begin
Copyright 2013, Chris Fullmer
All Rights Reserved

Disclaimer
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

License
This software is distributed under the Smustard End User License Agreement
http://www.smustard.com/eula

Information
Author - Chris Fullmer
Organization - www.ChrisFullmer.com and distributed on the SketchUp Extension Warehouse and smustard.com
Name - Camera Recorder
SU Version - 2013, 8, 7

Description
This script records the navigating of a model and exports it out as a series of images to be turned into a movie using quicktime or any other capable video editor.

Usage
It is installed in Plugins > Chris Fullmer Tools > Camera Recorder > (items).  Start recording and navigate the model.  Then stop recording.  Clear the recording if you desire.  Or export the animation into an existing folder.  Supports jpg and png file types.

History::
1.0:: 2009-11-10
   * Original release.  
1.5:: 2009-11-11
   * Fixed the lag bug that occured in the beginning of each recording
	* Added more user feedback when buttons are pressed in the wrong order
	* Added the abilty to control image output height, width and anti-aliasing
	* Added better file names so that more software will correctly order the image frames
	* Added a recording playback button, so that recordings can be played back before exporting.
	* Added a tool to place constructions points at the location of each frame, with another construction point where the camera is pointing.
	* Added some new icons, but they still need help.
1.7:: 2009-11-17
	* Added the ability to export frames to scenes
	* Script now remembers user settings from SketchUp session to session
1.7.1 2013-05-07
  * Compatible with EW
  * Cleaned up all global variables and namespaces
1.7.2 2013-05-08
  * Fixed an error with extension double loading
=end


module CLF_Extensions_NS
  
  module CLF_Camera_Recorder            #EDIT
    require 'sketchup.rb'
    require 'extensions.rb'
    
    NAME = "clf_camera_recorder"        #EDIT
    UNAME = "CLF Camera Recorder"       #EDIT
    MENU_NAME = "Camera Recorder"       #EDIT
    version = "1.7.2"           #EDIT
    desc = "This plugin will record model navigation out to a series of images or scenes, which can then be turned into a movie."
    copy_year = "2013"
    author = "Chris Fullmer"
    
#------edit above--------------------------------------------------------------- 

    
    extension = SketchupExtension.new UNAME, NAME+"/"+NAME+"_menus.rb"

    #The name= method sets the name which appears for an extension inside the Extensions Manager dialog.
    extension.name = UNAME

    # The description= method sets the long description which appears beneath an extension inside the Extensions Manager dialog.
    extension.description = desc + "  Access it via Plugins > Chris Fullmer Tools > "+MENU_NAME

    # The version method sets the version which appears beneath an extension inside the Extensions Manager dialog.
    extension.version = version

    # Create an entry in the Extension list that loads a script called
    # stairTools.rb.
    extension.copyright = copy_year
     
    # The creator= method sets the creator string which appears beneath an extension inside the Extensions Manager dialog.
    extension.creator = author

    # The register_extension method is used to register an extension with SketchUp's extension manager (in SketchUp preferences).
    Sketchup.register_extension( extension, true )
    
  end
end
