# See the loader file for license and author info

require 'sketchup.rb'
module CLF_Extensions_NS
	module CLF_Camera_Recorder

    class Clf_ViewObs < Sketchup::ViewObserver
      #@@cam_obs = nil
      @@eye = [] 
      @@direction = [] 
      @@up = [] 
      @@recording = false
      @@pts_exist = false
      @@path = File.join(File.dirname(__FILE__), "EXPORT")
      @@pc = Object::RUBY_PLATFORM.include?("mswin")
      @@a = Sketchup.version
      @@b = @@a[0,1].to_i
      if @@b > 6
        @@fast = true
      else
        @@fast = false
      end
      
      #---------Read Defaults, the 3rd value is only used if there is no default already present----------
      @@fps = Sketchup.read_default( 'clf_camera_recorder', 'fps', '15' )
      @@export_type = Sketchup.read_default( 'clf_camera_recorder', 'export_type', 'Scenes' )
      @@destination_folder = Sketchup.read_default( 'clf_camera_recorder', 'destination_folder', @@path )
      @@destination_folder.gsub!("/", "\\\\") if @@pc
      @@base_file_name = Sketchup.read_default( 'clf_camera_recorder', 'base_file_name', 'image' )
      @@file_type = Sketchup.read_default( 'clf_camera_recorder', 'file_type', 'png' )
      @@image_width = Sketchup.read_default( 'clf_camera_recorder', 'image_width', '800' )
      @@image_height = Sketchup.read_default( 'clf_camera_recorder', 'image_height', '600' )
      @@image_anti_alias = Sketchup.read_default( 'clf_camera_recorder', 'image_anti_alias', 'Yes' )
      @@first_run = Sketchup.read_default( 'clf_camera_recorder', 'first_run', 'yes' )

      #----------Write defaults, will re-write any existing defaults------------
      if @@first_run == 'yes'
        Sketchup.write_default( 'clf_camera_recorder', 'first_run', 'no' )
        Sketchup.write_default( 'clf_camera_recorder', 'fps', @@fps )
        Sketchup.write_default( 'clf_camera_recorder', 'export_type', @@export_type )
        Sketchup.write_default( 'clf_camera_recorder', 'destination_folder', @@destination_folder.gsub( /\\/, "/" ) )
        Sketchup.write_default( 'clf_camera_recorder', 'base_file_name', @@base_file_name )
        Sketchup.write_default( 'clf_camera_recorder', 'file_type', @@file_type )
        Sketchup.write_default( 'clf_camera_recorder', 'image_width', @@image_width )
        Sketchup.write_default( 'clf_camera_recorder', 'image_height', @@image_height )
        Sketchup.write_default( 'clf_camera_recorder', 'image_anti_alias', @@image_anti_alias )
      end
      

      
      def export_images
        prompt = ["Destination Folder", "Base File Name", "File Type", "Image Width", "Image Height", "Anti-Alias?"]
        value = [@@destination_folder, @@base_file_name, @@file_type, @@image_width.to_i, @@image_height.to_i, @@image_anti_alias]
        list = ["","","png|jpg", "", "", "Yes|No"]
        results = UI.inputbox( prompt, value, list, "Export Options" )
        if results
          @@destination_folder = results[0]
          puts results[0]
          @@base_file_name = results[1]
          full_file_name = @@base_file_name+"00000"
          @@file_type = results[2]
          @@image_width = results[3]
          @@image_height = results[4]
          @@image_anti_alias = results[5]
          Sketchup.write_default( 'clf_camera_recorder', 'destination_folder', @@destination_folder.gsub( /\\/,"/" ) )
          Sketchup.write_default( 'clf_camera_recorder', 'base_file_name', @@base_file_name.to_s )
          Sketchup.write_default( 'clf_camera_recorder', 'file_type', @@file_type )
          Sketchup.write_default( 'clf_camera_recorder', 'image_width', @@image_width )
          Sketchup.write_default( 'clf_camera_recorder', 'image_height', @@image_height )
          Sketchup.write_default( 'clf_camera_recorder', 'image_anti_alias', @@image_anti_alias )
          if @@image_anti_alias == "Yes"
            image_aa = true
          else
            image_aa = false
          end
          puts "Exporting"
          view = Sketchup.active_model.active_view
          
          @@eye.each_index do |e|
            Sketchup.status_text = "Exporting #{e} of #{@@eye.length} images."
            cam = cam_maker( e )
            view.camera=cam
            img_full_name = File.join( @@destination_folder, full_file_name+"."+@@file_type )
            view.write_image( img_full_name, @@image_width, @@image_height, image_aa )
            full_file_name=full_file_name.next
          end
          Sketchup.status_text = "Done Exporting!"
          puts "Done!"
        else
          puts "Cancelled"
        end
      end
      
      
      def record
        if !@@recording
          if @@eye.length > 0
            cancel = clear_rec
            if cancel == "cancel"
              UI.messagebox "Recording Cancelled.  You must clear previous recording first."
              return
            end
          end
          prompts = ["How many FPS?"]
          defaults = [@@fps]
          results = UI.inputbox( prompts, defaults, "Camera Recorder" )
          if results
            @@recording = true
            @@fps = results[0]
            Sketchup.write_default( 'clf_camera_recorder', 'fps', @@fps.to_s )
            @@speed = 1.0/@@fps.to_f
            @@frame = 0
            @@time = Time.now
          else
            puts "Cancelled"
            Sketchup.active_model.active_view.remove_observer(self)
          end
        else
          Sketchup.status_text = "Already recording!"
        end
      end
      
      
      def stop_rec
        Sketchup.active_model.active_view.remove_observer(self)
        if @@eye.length > 0
          UI.messagebox @@eye.length.to_s + " frames recorded."
          @@recording = false
          Sketchup.status_text = "Recording Stopped!"
        else
          UI.messagebox "Nothing has been recorded.  Please press Record first."
        end
      end # method stop_rec
      
      # This clears the recording.  It first checks if there is a recording in memmory.
      # It then pops up a Yes or No messagebox if there is a recording, or alerts the user that
      # nothing had been recorded.  If they choose to clear the recording, it clears it.  Simple
      def clear_rec
        if @@eye.length > 0
          continue = UI.messagebox "Clear recording?", MB_YESNO
          if continue == 6
            @@time = []
            @@direction = []
            @@up = []
            @@eye = []
            @@pts_exist = false
            puts "cleared"
          else
            puts "clear cancelled"
            return "cancel"
          end
        else
          UI.messagebox "No recording to clear!"
        end
      end # Method
      
      
      def draw_curve
        Sketchup.active_model.start_operation "Add Curve", @@fast
        if @@eye.length > 0
          if @@pts_exist
            results = UI.messagebox "Points have already been created for this Animation, re-output them?", MB_YESNO
            if results == 6
              continue = true
            else
              continue = false
            end
          else
            continue = true
          end
        
          if continue
            model = Sketchup.active_model
            ents = model.entities
            g = ents.add_group
            gents = g.entities
            @@eye.each_index do |e| 
              gents.add_cpoint( @@eye[e] )
              @@direction[e].length = 100
              target = @@eye[e].clone.offset @@direction[e]
              gents.add_cpoint( target )
            end
            g.locked=true
            @@pts_exist = true
          else
            puts "recording existed, but re-export was cancelled."
          end
        else
          UI.messagebox "No recording to create points for."
        end
        Sketchup.active_model.commit_operation
      end
      
      
      def cam_maker( index )
        Sketchup::Camera.new( @@eye[index], @@direction[index], @@up[index] )
      end
      
      def make_scenes
        Sketchup.active_model.start_operation "Make Scenes from Frames", @@fast
        pages = Sketchup.active_model.pages
        view = Sketchup.active_model.active_view
        pg_name = "f"
        pg_num = 1
        @@eye.each_index do |index|
          cam = cam_maker( index )
          view.camera=cam
          page = pages.add pg_name+pg_num.to_s, 1
          pg_num = pg_num + 1
        end
        Sketchup.active_model.commit_operation
      end
      
      
      def animate
        if @@eye.length >0
          Sketchup.active_model.active_view.animation = self
        else
          UI.messagebox "There is no recording to replay."
        end
      end
      
      
      def nextFrame(view)
        cam = cam_maker( @@frame )
        view.camera = cam
        view.show_frame @@speed
        if @@frame+1 < @@eye.length
          @@frame = @@frame+1
          return true
        else
          @@frame = 0
          return false
        end
      end

      
      def export
        if @@eye.length > 0
          prompt = ["Export as: "]
          defaults = [@@export_type]
          values = ["Scenes|Images"]
          results = UI.inputbox( prompt, defaults, values, "Export Type" )
          @@export_type = results[0]
          Sketchup.write_default( 'clf_camera_recorder', 'export_type', @@export_type )
          make_scenes if @@export_type == "Scenes"
          export_images if @@export_type == "Images"
        else
          UI.messagebox "There is no recording to export."
        end
      end

      
      def onViewChanged(view)
          elapsed = Time.now - @@time
          missed_views = (elapsed/@@speed).to_i
          if elapsed >= @@speed
            missed_views.times do
              @@eye << view.camera.eye
              @@direction << view.camera.direction
              @@up << view.camera.up
              @@time = Time.now
            end
          end
      end # method

    end # Class
    
    @@cam_obs = nil
     
    def self.get_obs
      if @@cam_obs
        return @@cam_obs
      else
        @@cam_obs = Clf_ViewObs.new
        return @@cam_obs
      end
    end

  end
end





